/*
 * For use with the SmallEiffel adaptation of Michael Ravits' Eiffel wrapper
 * for the MySQL database
 *
 * Original (C) 1999 by Michael Ravits, for use with ISE Eiffel
 *   http://www.eiffel-forum.org/archive/ravits/mysql.htm
 *   http://www.netpedia.net/hosting/newplayer/
 *
 * Adapted 2000/07 by Michael Peceny (peceny@web.de) for use with SmallEiffel
 *
 * Provides the functions for the actual interaction with the MySQL database,
 * which are called as external C functions by the MYSQL_DATABASE Eiffel class.
 * See the documentation on how to use.
 */

#include <stdio.h>
#include <stdlib.h>
#include <mysql.h>

#define EIF_FALSE 0
#define EIF_TRUE 1

void *mb_get_new_mysql_pointer()
{
	void	*a;
	
	a = malloc( sizeof(MYSQL) );

	return a;
}

void *mb_get_new_mysql_result_pointer()
{
	void	*a;
	
	a = malloc( sizeof(MYSQL_RES) );

	return a;
}

void mb_init( MYSQL **mysql, MYSQL_RES **res )
{
	*res = NULL;
}

int mb_connect( MYSQL **mysql, char *host, char *username, char *password )
{
	if( mysql_connect(*mysql, host, username, password) == NULL )
	{
		return EIF_FALSE;
	}
	return EIF_TRUE;
}

void mb_disconnect( MYSQL **mysql )
{
	mysql_close( *mysql );
}

int mb_select_database( MYSQL **mysql,  char *db_name )
{
	if( mysql_select_db(*mysql, db_name) )
		return EIF_FALSE;
	
	return EIF_TRUE;
}

void mb_free_result_data( MYSQL_RES **res )
{
	mysql_free_result( *res );
}

void mb_free_mysql_pointer( MYSQL **mysql )
{
	free( *mysql );
}

int mb_execute_query( MYSQL **mysql, MYSQL_RES **res, char *query )
{
	if( *res != NULL )
	{
		mb_free_result_data( res );
		*res = NULL;
	}

	if( mysql_query(*mysql, query) == 0 )
	{
		*res = mysql_store_result( *mysql );

		if( res == NULL )
		{
			if( mysql_num_fields(*mysql) == 0 )
				return EIF_TRUE;
			else
				return EIF_FALSE;
		}
		else
			return EIF_TRUE;
	}
	else
		return EIF_FALSE;

	return EIF_FALSE;
}

int mb_row_count( MYSQL_RES **res )
{
	if( *res == NULL )
		return 0;
	
	return	mysql_num_rows( *res );
}

int mb_field_count( MYSQL_RES **res )
{
	if( *res == NULL )
		return 0;

	return mysql_num_fields( *res );
}

void *mb_field_name( MYSQL_RES **res, int n ) // char *
{
	MYSQL_FIELD	*field;

	if( *res == NULL )
		return "";
	
	mysql_field_seek( *res, n );
	field = mysql_fetch_field( *res );
	
	if( field == NULL )
		return "";

	return field->name;
}

int mb_field_size( MYSQL_RES **res, int n )
{
	MYSQL_FIELD	*field;
	
	if( *res == NULL )
		return 0;
	
	mysql_field_seek( *res, n );
	field = mysql_fetch_field( *res );

	if( field == NULL )
		return 0;
	
	return field->length;
}

void mb_select_row( MYSQL_RES **res, int n )
{
	mysql_data_seek( *res, n );
}

void *mb_get_row( MYSQL_RES **res )
{
	return (void *)mysql_fetch_row( *res );
}

void *mb_get_field( MYSQL_ROW row, int n ) // char *
{
	return row[n];
}

void *mb_get_mysql_error( MYSQL **mysql ) // char *
{
	return	mysql_error(*mysql);
}
